/*
    compile with: grep ^gcc 5bitstat.c  | sh
gcc -o 5bitstat 5bitstat.c -lm
*/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>

/* most .wav files: header size 44 */

void _5bitstat(char *read, unsigned bits, unsigned size, unsigned stride);
void reldev(unsigned *ary, unsigned size);
void *load_file(const char *filename, long *filesize);

int main(int argc, char **argv)
{
    char *buf, *read;
    long size;
    unsigned skip, bits, stride, channels, ch;

    if( argc < 4 || argc > 5 || !isdigit(*argv[2]) || !isdigit(*argv[3]) ||
                                        (argc == 5 && !isdigit(*argv[4])) ) {
        printf("usage: 5bitstat <file> <skip>[.<bits>] <stride> [<channels>]\n");
        return 1;
    }
    buf= load_file(argv[1], &size);
    if( !buf )
        return 2;
    skip= strtoul(argv[2], &read, 0);
    if( *read == '.' && isdigit(read[1]) )
        bits= strtoul(read+1, NULL, 0);
    else
        bits= 5;
    stride= strtoul(argv[3], NULL, 0);
    channels= argc == 5 ? strtoul(argv[4], NULL, 0) : 1;
    size -= skip;
    for( ch= 0; ch< channels; ++ch ) {
        printf("channel %u:\n", ch);
        _5bitstat(buf+skip+ch*stride/channels, bits, size, stride);
    }
    free(buf);
}


void _5bitstat(char *read, unsigned bits, unsigned size, unsigned stride)
{
    unsigned count[256];
    unsigned i, mask;

    mask= (1 << bits) - 1;
    for( i= 0; i <= mask; ++i )
        count[i]= 0;
    for( i= 0; i < size; i += stride )
        ++count[(unsigned char)read[i] & mask];
    for( i= 0; i <= mask; ++i )
        printf("%d\t%d\n", i, count[i]);
    reldev(count, mask+1);
}


void reldev(unsigned *ary, unsigned size)
{
    unsigned long long sum, sqrsum;
    unsigned i;

    sum= sqrsum= 0;
    for( i= 0; i< size; ++i ) {
        sum += ary[i];
        sqrsum += (unsigned long long)ary[i] * ary[i];
    }
    printf("RMS deviation relative to mean: %g\n",
            sqrt(size * (double)sqrsum / (sum*sum) - 1.0));
}


/*!
  \brief Read a file into RAM

  Allocates RAM and reads all of a file into it.  A zero byte is appended
  (useful for text files), and the size is written to the second argument.
  If unsuccessful, file error messages go into stderr.

  \param filename File name
  \param filesize Pointer to long variable for file/buffer size (or NULL)

  \return Pointer to file data, or NULL if the operation failed.
 */
void *load_file(const char *filename, long *filesize)
{
    FILE *in;
    void *buf;
    long size;

    in= fopen(filename, "r");
    if( !in ) {
        fprintf(stderr, "Could not open `%s' for reading.\n", filename);
        return NULL;
    }
    fseek(in, 0, SEEK_END);
    size= ftell(in);
    fseek(in, 0, SEEK_SET);
    buf= malloc(size+1);
    if( !buf ) {
        fprintf(stderr, "Could not allocate memory for loading `%s'.\n", filename);
        fclose(in);
        return NULL;
    }
    if( ferror(in) || fread(buf, 1, size, in) != size ) {
        fprintf(stderr, "Error reading `%s'.\n", filename);
        free(buf);
        fclose(in);
        return NULL;
    }
    fclose(in);
    ((char*)buf)[size]= 0;
    if( filesize )
        *filesize= size;
    return buf;
}


