/*
    compile with: grep ^gcc bit0stat.c  | sh
gcc -o bit0stat bit0stat.c
*/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

/* most .wav files: header size 44 */

void bitstat(char *read, unsigned bit, unsigned size, unsigned stride);
void *load_file(const char *filename, long *filesize);

int main(int argc, char **argv)
{
    char *buf, *read;
    long size;
    unsigned skip, bitnr, stride, channels, ch;

    if( argc < 4 || argc > 5 || !isdigit(*argv[2]) || !isdigit(*argv[3]) ||
                                        (argc == 5 && !isdigit(*argv[4])) ) {
        printf("usage: bit0stat <file> <skip>[.<bit>] <stride> [<channels>]\n");
        return 1;
    }
    buf= load_file(argv[1], &size);
    if( !buf )
        return 2;
    skip= strtoul(argv[2], &read, 0);
    if( *read == '.' && isdigit(read[1]) )
        bitnr= strtoul(read+1, NULL, 0);
    else
        bitnr= 0;
    stride= strtoul(argv[3], NULL, 0);
    channels= argc == 5 ? strtoul(argv[4], NULL, 0) : 1;
    size -= skip;
    for( ch= 0; ch< channels; ++ch ) {
        printf("bit %d, channel %u:\n", bitnr, ch);
        bitstat(buf+skip+ch*stride/channels, bitnr, size, stride);
    }
    free(buf);
}


void bitstat(char *read, unsigned bit, unsigned size, unsigned stride)
{
    unsigned count[2];
    unsigned i;

    count[0]= count[1]= 0;
    for( i= 0; i < size; i += stride )
        ++count[(read[i] >> bit) & 1];
    printf("#1 / #0 - 1 = %d / %d - 1 = %.16g\n", count[1], count[0], (double)count[1]/(double)count[0] - 1.0);
}


/*!
  \brief Read a file into RAM

  Allocates RAM and reads all of a file into it.  A zero byte is appended
  (useful for text files), and the size is written to the second argument.
  If unsuccessful, file error messages go into stderr.

  \param filename File name
  \param filesize Pointer to long variable for file/buffer size (or NULL)

  \return Pointer to file data, or NULL if the operation failed.
 */
void *load_file(const char *filename, long *filesize)
{
    FILE *in;
    void *buf;
    long size;

    in= fopen(filename, "r");
    if( !in ) {
        fprintf(stderr, "Could not open `%s' for reading.\n", filename);
        return NULL;
    }
    fseek(in, 0, SEEK_END);
    size= ftell(in);
    fseek(in, 0, SEEK_SET);
    buf= malloc(size+1);
    if( !buf ) {
        fprintf(stderr, "Could not allocate memory for loading `%s'.\n", filename);
        fclose(in);
        return NULL;
    }
    if( ferror(in) || fread(buf, 1, size, in) != size ) {
        fprintf(stderr, "Error reading `%s'.\n", filename);
        free(buf);
        fclose(in);
        return NULL;
    }
    fclose(in);
    ((char*)buf)[size]= 0;
    if( filesize )
        *filesize= size;
    return buf;
}


