/*
    compile with: grep ^gcc bitcorr.c  | sh
gcc -o bitcorr bitcorr.c -lm
*/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>

/* most .wav files: header size 44 */

void bitautocorr(char *read, unsigned bit, unsigned size, unsigned stride);
void *load_file(const char *filename, long *filesize);

int main(int argc, char **argv)
{
    char *buf, *read;
    long size;
    unsigned skip, bitnr, stride, channels, ch;

    if( argc < 4 || argc > 5 || !isdigit(*argv[2]) || !isdigit(*argv[3]) ||
                                        (argc == 5 && !isdigit(*argv[4])) ) {
        printf("usage: bitcorr <file> <skip>[.<bit>] <stride> [<channels>]\n");
        return 1;
    }
    buf= load_file(argv[1], &size);
    if( !buf )
        return 2;
    skip= strtoul(argv[2], &read, 0);
    if( *read == '.' && isdigit(read[1]) )
        bitnr= strtoul(read+1, NULL, 0);
    else
        bitnr= 0;
    stride= strtoul(argv[3], NULL, 0);
    channels= argc == 5 ? strtoul(argv[4], NULL, 0) : 1;
    size -= skip;
    for( ch= 0; ch< channels; ++ch ) {
        printf("channel %u:\n", ch);
        bitautocorr(buf+skip+ch*stride/channels, bitnr, size, stride);
    }
    free(buf);
}


#define MAXOFFSET   1024

void bitautocorr(char *read, unsigned bit, unsigned size, unsigned stride)
{
    double corr, min, max, sqrsum, rms;
    unsigned offset, maxoff, conv, i;

    maxoff= MAXOFFSET*stride;
    if( maxoff >= size )
        maxoff= size - stride;
    size -= stride;
    min= 1.0;
    max= 0.0;
    sqrsum= 0.0;
    for( offset= stride; offset < maxoff; offset += stride, size -= stride ) {
        conv= 0;
        for( i= 0; i < size; i += stride )
            conv += ~((read[i]^read[i+offset]) >> bit) & 1;
        corr= 2.0 * ((double)conv/(size/stride) - 0.5);
        printf("offset %4d:\t%g\n", offset/stride, corr);
        sqrsum += corr*corr;
        if( min > fabs(corr) )
            min= fabs(corr);
        if( max < fabs(corr) )
            max= fabs(corr);
    }
    rms= sqrt(sqrsum / ((maxoff - 1) / stride));
    printf("RMS: %g  Minabs: %g  Maxabs: %g\n", rms, min, max);
}


/*!
  \brief Read a file into RAM

  Allocates RAM and reads all of a file into it.  A zero byte is appended
  (useful for text files), and the size is written to the second argument.
  If unsuccessful, file error messages go into stderr.

  \param filename File name
  \param filesize Pointer to long variable for file/buffer size (or NULL)

  \return Pointer to file data, or NULL if the operation failed.
 */
void *load_file(const char *filename, long *filesize)
{
    FILE *in;
    void *buf;
    long size;

    in= fopen(filename, "r");
    if( !in ) {
        fprintf(stderr, "Could not open `%s' for reading.\n", filename);
        return NULL;
    }
    fseek(in, 0, SEEK_END);
    size= ftell(in);
    fseek(in, 0, SEEK_SET);
    buf= malloc(size+1);
    if( !buf ) {
        fprintf(stderr, "Could not allocate memory for loading `%s'.\n", filename);
        fclose(in);
        return NULL;
    }
    if( ferror(in) || fread(buf, 1, size, in) != size ) {
        fprintf(stderr, "Error reading `%s'.\n", filename);
        free(buf);
        fclose(in);
        return NULL;
    }
    fclose(in);
    ((char*)buf)[size]= 0;
    if( filesize )
        *filesize= size;
    return buf;
}


