/*
    compile with: grep ^gcc xor.c  | sh
gcc -o xor xor.c
*/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <stdarg.h>

#define BUFSIZE     1024
#define BUFBYTES    (BUFSIZE * sizeof(unsigned))

void usageexit(void);
void ferrexit(FILE *handle, const char *format, ...);

int main(int argc, char **argv)
{
    FILE *src1, *src2, *dest;
    unsigned buf1[BUFSIZE], buf2[BUFSIZE];
    long offset;
    unsigned bytes1, bytes2, count, wrapwarned;

    if( argc != 5 || !isdigit(*argv[1]) )
        usageexit();
    offset= strtol(argv[1], NULL, 0);
    src1= fopen(argv[2], "rb");
    ferrexit(src1, "Error opening `%s' for reading.\n", argv[2]);
    src2= fopen(argv[3], "rb");
    ferrexit(src2, "Error opening `%s' for reading.\n", argv[3]);
    fseek(src1, offset, SEEK_SET);
    ferrexit(src1, "Could not seek to position %ld in `%s'.\n", offset, argv[2]);
    dest= fopen(argv[4], "wb");
    ferrexit(dest, "Could not create / truncate `%s'.\n", argv[4]);
    while( 0 < (bytes2= fread(buf2, 1, BUFBYTES, src2)) )
    {
        bytes1= fread(buf1, 1, bytes2, src1);
        ferrexit(src1, "Error reading from `%s'.\n", argv[2]);
        if( bytes1 != bytes2 ) {
            fseek(src1, 0, SEEK_SET);
            fread(buf1 + bytes1, 1, bytes2-bytes1, src1);
            ferrexit(src1, "Error reading from `%s'.\n", argv[2]);
        }
        for( count= 0; count< BUFSIZE; ++count )
            buf2[count] ^= buf1[count];
        fwrite(buf2, 1, BUFBYTES, dest);
        ferrexit(dest, "Error writing `%s'.\n", argv[4]);
    }
    ferrexit(src2, "Error reading from `%s'.\n", argv[3]);
    fclose(dest);
    fclose(src2);
    fclose(src1);
    return 0;
}


void usageexit(void)
{
    printf("usage: xor <offset> <source1> <source2> <dest>\n"
"Bitwise exclusive ORs part of <source1> with <source2> and writes the result\n"
"to <dest>.  <offset> indicates where in <source1> to start; if the end of\n"
"<source1> is hit, a warning is printed and <source1> is reread from the start.\n"
"The new read offset in <source1> is always printed.\n");
    exit(0);
}


void ferrexit(FILE *handle, const char *format, ...)
{
    va_list ap;

    if( handle && !ferror(handle) )
        return;
    perror("");
    if( format ) {
        va_start(ap, format);
        vfprintf(stderr, format, ap);
        va_end(ap);
    }
    exit(1);
}

